package com.fr.plugin.decision.integration.controller;

import com.fr.base.ServerConfig;
import com.fr.decision.authority.data.User;
import com.fr.decision.webservice.CrossDomainResponse;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.io.utils.ResourceIOUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.integration.config.IntegrateConf;
import com.fr.plugin.decision.integration.service.IntegrateAuthService;
import com.fr.plugin.decision.integration.service.IntegrateCustomService;
import com.fr.plugin.decision.integration.utils.CommonUtils;
import com.fr.plugin.decision.integration.utils.Constants;
import com.fr.plugin.decision.integration.utils.LogUtils;
import com.fr.stable.StableUtils;
import com.fr.stable.StringUtils;
import com.fr.third.fasterxml.jackson.databind.ObjectMapper;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.*;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping({"/third/auth"})
@LoginStatusChecker(required = false)
public class IntegratedController {

    private ObjectMapper mapper = new ObjectMapper();

    /**
     * 建议用于后台换取 fine_auth_token
     * @param request
     * @param response
     * @param body 请求体
     * @param third_token 请求参数
     * @return  fine_auth_token
     * @throws Exception
     */
    @RequestMapping("/login")
    @ResponseBody
    public Response loginByToken(HttpServletRequest request, HttpServletResponse response, @RequestBody(required = false) Map body, @RequestParam(required = false, value = Constants.TOKEN_NAME) String third_token) throws Exception {
        String token = third_token;
        if (StringUtils.isBlank(third_token)) {
            token = (String) body.get(Constants.TOKEN_NAME);
        }
        LogUtils.info("/login request start:{} token:{}", System.currentTimeMillis(), third_token);
        String username = IntegrateAuthService.getInstance().getUserNameFromToken(token);
        String fineAuthToken = LoginService.getInstance().login(request, response, username);
        LogUtils.info("/login request end:{}", System.currentTimeMillis());
        return Response.ok(fineAuthToken);
    }

    /**
     * 用于前台跨域请求
     * @param request
     * @param response
     * @param third_token
     * @param callback
     * @return
     */
    @RequestMapping(
            value = {"/cross/login"},
            method = {RequestMethod.GET},
            produces = "application/javascript; charset=utf-8"
    )
    @ResponseBody
    public String crossLoginByToken(HttpServletRequest request,
                                    HttpServletResponse response,
                                    @RequestParam(Constants.TOKEN_NAME) String third_token,
                                    @RequestParam(value = "callback", required = false, defaultValue = "callback") String callback
    ) {
        try {
            LogUtils.info("/cross/login request start:{} token:{}", System.currentTimeMillis(), third_token);
            String username = IntegrateAuthService.getInstance().getUserNameFromToken(third_token);
            String fineAuthToken = LoginService.getInstance().login(request, response, username);
            String domainResponse = CrossDomainResponse.create().callbackFuncName(callback)
                    .parameter("accessToken", fineAuthToken)
                    .parameter("url", IntegrateCustomService.getInstance().generateDefaultHomePageUrl(request))
                    .parameter("status", "success")
                    .createCrossDomainResponse();
            LogUtils.info("/cross/login request end:{}", System.currentTimeMillis());
            return domainResponse;
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e, e.getMessage());
            return CommonUtils.getStackTraceInfo(e);
        }
    }

    @RequestMapping(
            value = {"/login/iframe"},
            method = {RequestMethod.GET}
    )
    @ResponseBody
    public String loginByIframe(HttpServletRequest request,
                                    HttpServletResponse response,
                                    @RequestParam(Constants.TOKEN_NAME) String third_token) throws Exception {
        String username = IntegrateAuthService.getInstance().getUserNameFromToken(third_token);
        String token = LoginService.getInstance().login(request, response, username);
        return WebServiceUtils.parseWebPageResource("com/fr/plugin/decision/integration/login.html", new HashMap<String, Object>() {{
            put("token", token);
        }});
    }

    /**
     * URL设计器启动需要用的接口
     */

    /**
     * 解析获取 token 对应的用户名
     * @param third_token
     * @return
     */
    @RequestMapping(
            value = {"/username"}
    )
    @ResponseBody
    public String getUsernameFromToken(@RequestParam(Constants.TOKEN_NAME) String third_token) throws Exception {
        return IntegrateAuthService.getInstance().getUserNameFromToken(third_token);
    }

    /**
     * 获取用户对应的token，用于web唤醒设计器传入token
     */
    @RequestMapping(
            value = {"/username/token"}
    )
    @ResponseBody
    @LoginStatusChecker
    public Response getThirdTokenByUsername(HttpServletRequest request) throws Exception {
        String username = LoginService.getInstance().getCurrentUserNameFromRequest(request);
        return Response.ok(IntegrateConf.getInstance().getWebStartedInfo(username));
    }

    /**
     * 登录状态下
     */
    @RequestMapping(
            value = {"/designer/start"}
    )
    @ResponseBody
    //@LoginStatusChecker(tokenResource = TokenResource.COOKIE)
    public String openDesignerStartUrl(HttpServletRequest request,
                                       HttpServletResponse response,
                                       @RequestParam(value = "path", required = false) String path) throws Exception {
        String username = LoginService.getInstance().getCurrentUserNameFromRequestCookie(request);
        if (StringUtils.isEmpty(username)) return StringUtils.EMPTY;
        response.setContentType("text/html; charset=" + ServerConfig.getInstance().getServerCharset());
        return IntegrateConf.getInstance().getWebStartURL(username, path);
    }

    /**
     *  插件配置读取、保存
     */
    @RequestMapping(
            value = {"/conf"},
            method = RequestMethod.GET
    )
    @ResponseBody
    @LoginStatusChecker
    public Response getIntegrateConfig(HttpServletRequest request, HttpServletResponse response) throws Exception {
        String userId = UserService.getInstance().getCurrentUserIdFromCookie(request);
        if (UserService.getInstance().isAdmin(userId)) {
            response.setContentType("application/json;charset=" + ServerConfig.getInstance().getServerCharset());
            mapper.writeValue(response.getOutputStream(), IntegrateConf.getInstance());
        }

        return Response.error("getIntegrateConf", "仅管理员用户可见");
    }

    @RequestMapping(
            value = {"/conf"},
            method = RequestMethod.POST
    )
    @ResponseBody
    @LoginStatusChecker
    public Response updateIntegrateConfig(HttpServletRequest request) throws IOException {
        IntegrateConf conf = mapper.readValue(request.getInputStream(), IntegrateConf.class);
        IntegrateAuthService.getInstance().updateIntegrateConf(conf);
        return Response.success();
    }


    /**
     * 插件历史接口，做兼容使用
     */

    @RequestMapping(
            value = {"/tpl/paste"}
    )
    @ResponseBody
    @Deprecated
    public Response pasteTemplate(@RequestParam(Constants.TOKEN_NAME) String third_token, @RequestParam("tplPath") String tplPath) throws Exception {
        String username = IntegrateAuthService.getInstance().getUserNameFromToken(third_token);
        if (StringUtils.isBlank(username))
            return Response.error("pasteTemplate", "TOKEN解析异常");

        if (StringUtils.isBlank(tplPath) || !ResourceIOUtils.exist(StableUtils.pathJoin("reportlets", tplPath)))
            return Response.error("pasteTemplate", "模板文件不存在");

        ResourceIOUtils.copy(
                StableUtils.pathJoin("reportlets", tplPath),
                StableUtils.pathJoin("reportlets", username, tplPath.substring(tplPath.lastIndexOf("/")))
        );
        return Response.success();
    }

    @RequestMapping(
            value = {"/tpl/remote/list"}
    )
    @ResponseBody
    @Deprecated
    public Response getUserRemoteTplList(HttpServletRequest request, @RequestParam(Constants.TOKEN_NAME) String third_token) throws Exception {
        String username = IntegrateAuthService.getInstance().getUserNameFromToken(third_token);
        if (StringUtils.isBlank(username))
            return Response.error("UserRemoteTplList", "TOKEN解析异常");

        User user = UserService.getInstance().getUserByUserName(username);
        return Response.ok(IntegrateCustomService.getInstance().getRemoteTplListByUserId(user.getId(), WebUtils.createServletURL(request)));
    }

    /**
     * token 换取帆软token，主要用于后台请求
     */
    @RequestMapping({"/token"})
    @ResponseBody
    @Deprecated
    public String tokenExchangeFineAuthToken(HttpServletRequest request, HttpServletResponse response, @RequestBody Map body) throws Exception {
        String username = IntegrateAuthService.getInstance().getUserNameFromToken((String) body.get(Constants.TOKEN_NAME));
        return LoginService.getInstance().login(request, response, username);
    }

    /**
     * 处理下异常返回值统一格式返回
     * @param ex
     * @return
     */
    @ExceptionHandler(Exception.class)
    @ResponseBody
    public Response exceptionRespond(Exception ex) {
        FineLoggerFactory.getLogger().error(ex.getMessage(), ex);
        return CommonUtils.formatException(ex);
    }
}
