package com.fr.plugin.decision.integration.service;


import com.fr.cert.token.Claims;
import com.fr.cert.token.Jwts;
import com.fr.cert.token.SignatureAlgorithm;
import com.fr.decision.base.util.UUIDUtil;
import com.fr.decision.privilege.TransmissionTool;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.io.utils.ResourceIOUtils;
import com.fr.plugin.decision.integration.bean.user.UserInfoBean;
import com.fr.plugin.decision.integration.config.IntegrateConf;
import com.fr.plugin.decision.integration.config.strategy.CreatePwdStrategy;
import com.fr.plugin.decision.integration.config.strategy.user.impl.OnlyUserStrategy;
import com.fr.plugin.decision.integration.exception.JwtKeyNullException;
import com.fr.plugin.decision.integration.exception.JwtSubjectFormatException;
import com.fr.plugin.decision.integration.exception.TokenDecodeExpiredException;
import com.fr.plugin.decision.integration.exception.TokenNullException;
import com.fr.plugin.decision.integration.utils.AESUtils;
import com.fr.plugin.decision.integration.utils.CommonUtils;
import com.fr.plugin.decision.integration.utils.LogUtils;
import com.fr.plugin.decision.integration.utils.UserUtils;
import com.fr.report.ReportContext;
import com.fr.report.constant.RoleType;
import com.fr.report.data.RemoteDesignAuthority;
import com.fr.report.util.RemoteDesignAuthHelper;
import com.fr.report.util.RemoteUserInfo;
import com.fr.stable.StableUtils;
import com.fr.stable.StringUtils;
import com.fr.third.fasterxml.jackson.databind.ObjectMapper;
import com.fr.transaction.Configurations;
import com.fr.transaction.WorkerAdaptor;
import com.fr.web.service.RemoteDesignAuthorityDataService;

import javax.crypto.spec.SecretKeySpec;
import javax.xml.bind.DatatypeConverter;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;

public class IntegrateAuthService {

    private volatile static IntegrateAuthService service;
    private static Properties properties = new Properties();
    private IntegrateConf config = IntegrateConf.getInstance();
    private ObjectMapper mapper = new ObjectMapper();

    private IntegrateAuthService() {
    }

    public static IntegrateAuthService getInstance() {
        if (service == null) {
            synchronized (IntegrateAuthService.class) {
                if (service == null) {
                    service = new IntegrateAuthService();
                }
            }
        }
        return service;
    }

    public String getUserNameFromToken(String thirdToken) throws Exception {
        if (StringUtils.isEmpty(thirdToken)) {
            throw new TokenNullException();
        }
        String userInfo = decodeJwtToken(thirdToken, loadJwtKey());
        if (config.getAesEncryptSubject()) {
            userInfo = AESUtils.aesDecryptByBytes(Base64.getDecoder().decode(userInfo), config.getAesEncryptKey());
        }

        return addOrUpdateUserInfo(userInfo);
    }

    private String loadJwtKey() {
        String key = config.getJwtKey();
        if (StringUtils.isBlank(key)) {
            key = properties.getProperty("key");
        }

        if (StringUtils.isBlank(key)) {
            throw new JwtKeyNullException();
        }
        return key;
    }

    private String decodeJwtToken(String thirdToken, String key) {
        Claims claims = Jwts.parser()
                .setSigningKey(key)
                .parseClaimsJws(thirdToken)
                .getBody();
        Date date = claims.getExpiration();
        Date now = new Date();
        LogUtils.info("parse third_token result date:{} now:{}", date, now);
        if (date != null && date.after(now)) {
            return claims.getSubject();
        }
        throw new TokenDecodeExpiredException();
    }


    /**
     * 校验更新/创建用户信息
     *
     * @param userInfoStr
     * @return username
     * @throws Exception
     */
    private String addOrUpdateUserInfo(String userInfoStr) throws Exception {
        LogUtils.info("update user info start");
        // 未开启创建用户时，直接返回用户名不需要创建用户信息
        if (!config.getCreateUserTurnOn()) {
            if (CommonUtils.isJsonFormatStr(userInfoStr)) {
                return mapper.readValue(userInfoStr, UserInfoBean.class).getUsername();
            }
            return userInfoStr;
        }

        // 仅用户名
        if (config.getUserStrategy() instanceof OnlyUserStrategy) {
            if (CommonUtils.isJsonFormatStr(userInfoStr)) {
                throw new JwtSubjectFormatException();
            }
            // 用户不存在时基于用户名创建下用户
            if (!UserUtils.existUsername(userInfoStr)) {
                UserService.getInstance().addUser(buildBaseUserBean(userInfoStr));
                checkAndCreateRemoteDesignAuth(userInfoStr);
            }
            return userInfoStr;
        }

        // 含部门职位
        UserInfoBean userInfo = mapper.readValue(userInfoStr, UserInfoBean.class);
        // 管理员账号不进行处理
        if (UserUtils.isAdmin(userInfo.getUsername())) {
            return userInfo.getUsername();
        }
        UserBean userBean;
        if (!UserUtils.existUsername(userInfo.getUsername())) {
            UserService.getInstance().addUser(buildBaseUserBean(userInfo.getUsername()));
            LogUtils.info("create user success");
        }
        userBean = UserService.getInstance().getUserAccount(userInfo.getUsername());
        // 更新部门信息
        if (null != userInfo.getDepartment()) {
            userBean.setDepartmentPostIds(UserUtils.updateDepTreeAndPost(
                    userInfo.getDepartment().getName(),
                    userInfo.getDepartment().getId())
            );
            UserUtils.updateUserDepartmentPost(UserUtils.getAdminUserId(), userBean);
            LogUtils.info("update user department post success");
        }

        // 更新角色信息
        if (null != userInfo.getRoles()) {
            ArrayList<String> roles = new ArrayList<>();
            for (String roleName : userInfo.getRoles()) {
                roles.add(UserUtils.checkAndAddRole(roleName));
            }
            userBean.setRoleIds(roles.toArray(new String[0]));
            UserService.getInstance().updateUserRoles(UserUtils.getAdminUserId(), userBean);
            LogUtils.info("update user role success");
        }

        // 用户平台类型
        if (null != userInfo.getPlatformType()) {
            UserUtils.updateUserPlatformType(userBean, userInfo.getPlatformType());
            LogUtils.info("update user platform type success");
        }

        // 更新远程设计权限和目录信息
        checkAndCreateRemoteDesignAuth(userInfo.getUsername());
        LogUtils.info("update user info end");
        return userInfo.getUsername();
    }


    private UserBean buildBaseUserBean(String username) {
        UserBean userBean = new UserBean(UUIDUtil.generate(), username, username, true);
        String pwd;
        if (config.getUserStrategy().getPwdStrategy() == CreatePwdStrategy.SAME_USER) {
            pwd = username;
        } else {
            pwd = UserUtils.getRandomPassword(8);
        }
        userBean.setPassword(TransmissionTool.defaultEncrypt(pwd));
        return userBean;
    }

    /**
     * 校验如果用户没有自己同名模板跟目录文件夹的远程设计权限，则新建添加一下
     *
     * @param username
     */
    private void checkAndCreateRemoteDesignAuth(String username) throws Exception {
        LogUtils.info("check remote auth start");
        if (!config.getRemoteFolder()) return;
        RemoteUserInfo remoteUserInfo = RemoteDesignAuthHelper.getUserInfo(username);
        if (remoteUserInfo == null) {
            return;
        }
        String userRemoteFileName = formatUsername(username);
        List<RemoteDesignAuthority> remoteDesignAuthorities = RemoteDesignAuthorityDataService.getInstance().getAuthorities(remoteUserInfo.getUserID());
        for (RemoteDesignAuthority remoteDesignAuthority : remoteDesignAuthorities) {
            if (remoteDesignAuthority.getPathType() && StringUtils.equals(userRemoteFileName, remoteDesignAuthority.getPath())) {
                return;
            }
        }
        String path = StableUtils.pathJoin("reportlets", userRemoteFileName);
        if (!ResourceIOUtils.isDirectoryExist(path)) {
            ResourceIOUtils.createDirectory(path);
        }

        RemoteDesignAuthority authority = new RemoteDesignAuthority();
        authority.userName(username).userId(remoteUserInfo.getUserID()).path(userRemoteFileName).pathType(true).roleType(RoleType.USER);
        ReportContext.getInstance().getRemoteDesignAuthorityController().add(authority);
        LogUtils.info("check remote auth end");
    }

    private String formatUsername(String username) {
        return username.replaceAll("[\\\\/:*?\"<>|]", StringUtils.EMPTY);
    }

    public void updateIntegrateConf(IntegrateConf conf) {
        Configurations.update(new WorkerAdaptor(IntegrateConf.class) {
            @Override
            public void run() {
                IntegrateConf instance = IntegrateConf.getInstance();
                instance.setDecisionUrl(conf.getDecisionUrl());
                instance.setJwtKey(conf.getJwtKey());
                instance.setRemoteTokenAuth(conf.getRemoteTokenAuth());
                instance.setAesEncryptSubject(conf.getAesEncryptSubject());
                instance.setAesEncryptKey(conf.getAesEncryptKey());
                instance.setPortalURLTurnOn(conf.getPortalURLTurnOn());
                instance.setPortalURL(conf.getPortalURL());
                instance.setCrossDomain(conf.getCrossDomain());
                instance.setRemoteFolder(conf.getRemoteFolder());
                if (conf.getCreateUserTurnOn()) {
                    instance.setUserStrategy(conf.getUserStrategy());
                    instance.setCreateUserTurnOn(true);
                } else {
                    instance.setCreateUserTurnOn(false);
                }

                if (conf.getParaCheckTurnOn()) {
                    instance.setParaCheckUrl(conf.getParaCheckUrl());
                    instance.setParaCheckTurnOn(true);
                } else {
                    instance.setParaCheckTurnOn(false);
                }

                if (conf.getWebStartDesigner()) {
                    instance.setDownloadUrl32(conf.getDownloadUrl32());
                    instance.setDownloadUrl64(conf.getDownloadUrl64());
                    instance.setWebStartDesigner(true);
                } else {
                    instance.setWebStartDesigner(false);
                }
            }
        });
    }

    static {
        // 历史参数获取方式，兼容使用
        InputStream inputStream = ResourceIOUtils.read("/resources/third_auth.properties");
        if (null != inputStream) {
            try {
                properties.load(inputStream);
            } catch (IOException ignore) {}
        }
    }

}
