package com.fr.plugin.decision.integration.utils;

import com.finebi.cbb.utils.StringEscapeUtils;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.base.constant.type.operation.ManualOperationType;
import com.fr.decision.authority.controller.UserController;
import com.fr.decision.authority.data.*;
import com.fr.decision.authority.data.extra.user.type.PlatformUserKey;
import com.fr.decision.authority.data.extra.user.type.UserProductTypeKey;
import com.fr.decision.authority.data.personnel.DepRole;
import com.fr.decision.authority.entity.CustomRoleEntity;
import com.fr.decision.base.util.CollectionUtil;
import com.fr.decision.base.util.UUIDUtil;
import com.fr.decision.webservice.annotation.RoleCheckerType;
import com.fr.decision.webservice.bean.user.DepartmentPostBean;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.bean.user.UserUpdateBean;
import com.fr.decision.webservice.exception.user.AddUserProductTypeException;
import com.fr.decision.webservice.impl.user.type.UserProductType;
import com.fr.decision.webservice.utils.ControllerFactory;
import com.fr.decision.webservice.utils.UserSourceFactory;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.v10.user.DepartmentService;
import com.fr.decision.webservice.v10.user.PositionService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.general.ComparatorUtils;
import com.fr.plugin.decision.integration.config.IntegrateConf;
import com.fr.plugin.decision.integration.config.strategy.user.impl.ContainDepRoleStrategy;
import com.fr.stable.ArrayUtils;
import com.fr.stable.StringUtils;
import com.fr.stable.db.data.BaseDataRecord;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.restriction.Restriction;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.fr.stable.version.ProductVersion;
import com.fr.stable.version.ProductVersionManager;

import java.util.*;

public class UserUtils {

    public static final String POST_DEFAULT_NAME = "职员";
    public static final String DEP_ROOT_NAME = null;

    public static boolean existUsername(String username) throws Exception {
        return UserService.getInstance().getUserByUserName(username) != null;
    }

    public static String getAdminUserId() throws Exception {
        return UserService.getInstance().getAdminUserIdList().get(0);
    }

    public static boolean isAdmin(String username) throws Exception {
        return UserService.getInstance().getAdminUserNameList().contains(username);
    }

    public static List<String> updateDepTreeAndPost(String depName, String depId) throws Exception {
        ArrayList<String> result = new ArrayList<>();
        ContainDepRoleStrategy depStrategy = (ContainDepRoleStrategy) IntegrateConf.getInstance().getUserStrategy();
        switch (depStrategy.getDepStrategy()) {
            case DEP_ID_LENGTH:
                generateDepByIdLength(depName, depId);
                break;
            case DEP_NAME_GROUP:
                depId = generateDepByName(depName, depStrategy.getSeparator());
                break;
        }
        String postId = generateDefaultPosition(depId);
        result.add(WebServiceUtils.createUniquePostId(depId, postId));
        return result;
    }

    /**
     * 添加部门前先计算出最匹配的父节点，添加完之后在更新父节点下子节点的父信息
     *
     * @param depName
     * @param depId
     * @return
     * @throws Exception
     */
    private static String generateDepByIdLength(String depName, String depId) throws Exception {
        Department department = AuthorityContext.getInstance().getDepartmentController().getById(depId);
        if (department == null) {
            String depPId = getMostSimilarDepId(depId);
            if (checkDuplicatedDepartmentName(depPId, depName)) {
                depName = depName + "_" + depId;
            }
            addDepartment(depName, depId, depPId);
            // 更新depPId下子节点的父信息
            generateUnderParentDepartment(depPId, depId);
        }
        return depId;
    }

    /**
     * 根据部门名称拼接来更新部门
     *
     * @param depNameGroup
     * @return 部门id
     */
    private static String generateDepByName(String depNameGroup, String separator) throws Exception {
        String[] depArr = depNameGroup.split(StringEscapeUtils.escapeRegexSpecialWord(separator));
        String pId = DEP_ROOT_NAME;
        for (String depName : depArr) {
            Department department = getDepartmentByNameAndPid(depName, pId);
            if (department != null) {
                pId = department.getId();
            } else {
                String depId = UUIDUtil.generate();
                addDepartment(depName, depId, pId);
                pId = depId;
            }
        }
        return pId;
    }

    private static boolean existDepId(String depId) throws Exception {
        return AuthorityContext.getInstance().getDepartmentController().getById(depId) != null;
    }

    private static Department getDepartmentByNameAndPid(String depName, String pId) throws Exception {
        return AuthorityContext.getInstance().getDepartmentController().findOne(
                QueryFactory.create().addRestriction(RestrictionFactory.eq(Department.COLUMN_NAME, depName))
                        .addRestriction(RestrictionFactory.eq(Department.COLUMN_PARENT_ID, pId))
        );
    }

    private static boolean checkDuplicatedDepartmentName(String parentId, String depName) throws Exception {
        QueryCondition condition = QueryFactory.create().addRestriction(
                RestrictionFactory.and(new Restriction[]{RestrictionFactory.eq("name", depName), RestrictionFactory.eq("parentId", parentId)})
        );
        Department sameNameDep = AuthorityContext.getInstance().getDepartmentController().findOne(condition);
        return sameNameDep != null;
    }

    /**
     * 添加部门
     *
     * @param depName
     * @param depId
     * @param depPId
     * @throws Exception
     */
    private static void addDepartment(String depName, String depId, String depPId) throws Exception {
        Department record = (new Department()).id(depId).name(depName).parentId(depPId).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true);
        AuthorityContext.getInstance().getDepartmentController().add(record);
    }

    private static void generateUnderParentDepartment(String depPId, String depId) throws Exception {
        for (DepartmentPostBean departmentPostBean : DepartmentService.getInstance().getDepartmentsUnderParentDepartment(getAdminUserId(), depPId)) {
            if (ComparatorUtils.equals(depId, departmentPostBean.getId())) continue;
            String mostSimilarDepId = getMostSimilarDepId(departmentPostBean.getId());
            if (!ComparatorUtils.equals(mostSimilarDepId, depPId)) {
                Department record = AuthorityContext.getInstance().getDepartmentController().getById(departmentPostBean.getId());
                record.setParentId(mostSimilarDepId);
                AuthorityContext.getInstance().getDepartmentController().update(record);
            }
        }
    }

    private static List<Department> getAllDepartment() throws Exception {
        return AuthorityContext.getInstance().getDepartmentController().find(QueryFactory.create());
    }

    /**
     * 校验部门下是否有默认 POST_DEFAULT_NAME 职位
     *
     * @param departmentId
     * @return POST_DEFAULT_NAME 职位的 id
     * @throws Exception
     */
    private static String generateDefaultPosition(String departmentId) throws Exception {
        String adminId = UserService.getInstance().getAdminUserIdList().get(0);
        Post[] posts = ControllerFactory.getInstance().getPostController(adminId).getPositionsUnderParentDepartment(adminId, departmentId, null);
        for (Post post : posts) {
            if (ComparatorUtils.equals(post.getName(), POST_DEFAULT_NAME)) {
                return post.getId();
            }
        }
        String positionId = getDefaultPostId();
        AuthorityContext.getInstance().getPostController().addPostToDepartment(positionId, departmentId);
        return positionId;
    }

    /**
     * 校验是否有 POST_DEFAULT_NAME 职位，没有的话新建个
     *
     * @return POST_DEFAULT_NAME 职位的 id
     * @throws Exception
     */
    private static String getDefaultPostId() throws Exception {
        String adminId = UserService.getInstance().getAdminUserIdList().get(0);
        Post[] posts = ControllerFactory.getInstance().getPostController(adminId).getPositions(adminId, null);
        for (Post post : posts) {
            if (ComparatorUtils.equals(post.getName(), POST_DEFAULT_NAME)) {
                return post.getId();
            }
        }

        return PositionService.getInstance().addPosition(POST_DEFAULT_NAME).getId();
    }

    /**
     * 计算出与 depId 最相似的父节点
     *
     * @param depId
     * @return
     * @throws Exception
     */
    private static String getMostSimilarDepId(String depId) throws Exception {
        int similarLevel = 0;
        String mostSimilarDepId = DEP_ROOT_NAME;
        String specialDepRootId = ((ContainDepRoleStrategy) IntegrateConf.getInstance().getUserStrategy()).getSpecialRootId();
        if (StringUtils.isNotBlank(specialDepRootId)) {
            if (ComparatorUtils.equals(depId, specialDepRootId)) {
                return mostSimilarDepId;
            } else if (existDepId(specialDepRootId)) {
                mostSimilarDepId = specialDepRootId;
            }
        }
        for (Department department : getAllDepartment()) {
            String currentDepId = department.getId();
            // 部门id长度大于当前id长度的直接跳过，因为部门的树结构时根据id其实是否一致判断的
            if (currentDepId.length() > depId.length() || ComparatorUtils.equals(currentDepId, depId)) continue;
            int currentLevel = getDepIdSimilarityDegree(currentDepId, depId);
            if (currentLevel > similarLevel) {
                similarLevel = currentLevel;
                mostSimilarDepId = department.getId();
            }
        }
        return mostSimilarDepId;
    }

    private static int getDepIdSimilarityDegree(String depId1, String depId2) {
        String shortId, longId;
        if (depId1.length() > depId2.length()) {
            shortId = depId2;
            longId = depId1;
        } else {
            shortId = depId1;
            longId = depId2;
        }

        if (longId.startsWith(shortId)) {
            return shortId.length();
        }
        return 0;
    }

    public static String checkAndAddRole(String roleName) throws Exception {
        CustomRole customRole = AuthorityContext.getInstance().getCustomRoleController().findOne(
                QueryFactory.create().addRestriction(
                        RestrictionFactory.eq(CustomRoleEntity.COLUMN_NAME, roleName))
        );
        if (null != customRole) return customRole.getId();
        String roleId = UUIDUtil.generate();
        addRole(roleName, roleId);
        return roleId;
    }

    public static void addRole(String roleName, String roleId) throws Exception {
        String adminId = UserService.getInstance().getAdminUserIdList().get(0);
        CustomRole record = (new CustomRole()).id(roleId).name(roleName);
        ControllerFactory.getInstance().getCustomRoleController(adminId).addCustomRole(adminId, record);
    }

    /**
     * 更新用户的部门信息
     *
     * @param currentUserId
     * @param userBean
     * @throws Exception
     */
    public static void updateUserDepartmentPost(String currentUserId, UserBean userBean) throws Exception {
        if (userBean.getDepartmentPostIds() != null) {
            List<DepRole> newDepRoleList = new LinkedList();
            Iterator depRoleIt = userBean.getDepartmentPostIds().iterator();

            while (depRoleIt.hasNext()) {
                String depPostId = (String) depRoleIt.next();
                DepRole depRole = WebServiceUtils.parseUniqueDepartmentPostId(depPostId);
                newDepRoleList.add(AuthorityContext.getInstance().getDepartmentController().getDepRole(depRole.getDepartmentId(), depRole.getPostId()));
            }

            List<DepRole> oldDepRoleList = AuthorityContext.getInstance().getDepartmentController().findDepRoleByUser(userBean.getId());
            Map<String, DepRole> newIdDepRoleMap = CollectionUtil.convertToMap(newDepRoleList, BaseDataRecord::getId);
            Map<String, DepRole> oldIdDepRoleMap = CollectionUtil.convertToMap(oldDepRoleList, BaseDataRecord::getId);
            List<String> newDepRoleIdList = new LinkedList(newIdDepRoleMap.keySet());
            List<String> oldDepRoleIdList = new LinkedList(oldIdDepRoleMap.keySet());
            List<String> addDepRoleIdList = new LinkedList(newDepRoleIdList);
            List<String> removeDepRoleIdList = new LinkedList(oldDepRoleIdList);
            addDepRoleIdList.removeAll(oldDepRoleIdList);
            removeDepRoleIdList.removeAll(newDepRoleIdList);
            User targetUser = UserService.getInstance().getUserByUserId(userBean.getId());
            Iterator addDepRoleIt = addDepRoleIdList.iterator();

            String removeDepRoleId;
            DepRole depRole;
            while (addDepRoleIt.hasNext()) {
                removeDepRoleId = (String) addDepRoleIt.next();
                depRole = newIdDepRoleMap.get(removeDepRoleId);
                if (RoleCheckerType.DEPARTMENT.checkAuthority(currentUserId, depRole.getDepartmentId())) {
                    Department department = AuthorityContext.getInstance().getDepartmentController().getById(depRole.getDepartmentId());
                    Post post = AuthorityContext.getInstance().getPostController().getById(depRole.getPostId());
                    UserSourceFactory.getInstance().checkSource(targetUser, department);
                    UserSourceFactory.getInstance().checkSource(targetUser, post);
                    AuthorityContext.getInstance().getUserController().addUserToDepartmentAndPost(targetUser.getId(), depRole.getDepartmentId(), depRole.getPostId());
                }
            }

            addDepRoleIt = removeDepRoleIdList.iterator();

            while (addDepRoleIt.hasNext()) {
                removeDepRoleId = (String) addDepRoleIt.next();
                depRole = oldIdDepRoleMap.get(removeDepRoleId);
                if (RoleCheckerType.DEPARTMENT.checkAuthority(currentUserId, depRole.getDepartmentId())) {
                    AuthorityContext.getInstance().getUserController().removeUserFromDepartmentAndPost(targetUser.getId(), depRole.getDepartmentId(), depRole.getPostId());
                }
            }
        }
    }


    /**
     * 更新用户平台的类型
     *
     * @param userBean
     * @param platformTypeList
     * @throws Exception
     */
    public static void updateUserPlatformType(UserBean userBean, List<Integer> platformTypeList) throws Exception {
        UserController userController = AuthorityContext.getInstance().getUserController();
        // 查询用户的平台用户类型
        List<ExtraProperty> extraProperties = userController.findUserExtraProperty(QueryFactory.create().addRestriction(
                RestrictionFactory.eq(ExtraProperty.COLUMN_RELEATED_ID, userBean.getId())
        ).addRestriction(
                RestrictionFactory.startWith(ExtraProperty.COLUMN_NAME, UserProductTypeKey.KEY.getKey())
        ));

        adaptUserType(platformTypeList);
        List<Integer> newIdList = new LinkedList(platformTypeList);
        List<Integer> oldIdList = new LinkedList();
        for (ExtraProperty property : extraProperties) {
            UserProductTypeKey userProductTypeKey = (UserProductTypeKey) PlatformUserKey.fromKey(property.getName());
            oldIdList.add(UserProductType.fromKey(userProductTypeKey).toInteger());
        }

        // 校验新的用户平台类型是否冲突
        for (int i=0; i<platformTypeList.size(); i++) {
            UserProductType leftType = UserProductType.fromInteger(platformTypeList.get(i));
            for (int j=i+1; j<platformTypeList.size(); j++) {
                UserProductType rightType = UserProductType.fromInteger(platformTypeList.get(j));
                if (ArrayUtils.contains(leftType.excludeProductKeys(), rightType.transProductKey())) {
                    throw new IllegalArgumentException("add user platform type illegal " + platformTypeList);
                }
            }
        }

        LogUtils.debug("update user platform type, old type:{} new type:{}", oldIdList, newIdList);
        List<Integer> addIdList = new LinkedList(newIdList);
        List<Integer> removeIdList = new LinkedList(oldIdList);
        addIdList.removeAll(oldIdList);
        removeIdList.removeAll(newIdList);

        // 删除用户原有平台类型
        for (Integer i : removeIdList) {
            userController.removeUserProductType(userBean.getId(), UserProductType.fromInteger(i).transProductKey());
        }

        // 添加新增部分的平台类型
        out:
        for (Integer platformType : addIdList) {
            UserProductType userProductType = UserProductType.fromInteger(platformType);
            if (!userProductType.isNoLimitEditOpen()) {
                continue out;
            }

            if (userProductType.allowMax() > 0 && 1 > userProductType.leftRegisterTotal()) {
                throw new AddUserProductTypeException();
            }
            userController.addUserProductType(userBean.getId(), userProductType.transProductKey());
        }
    }

    /**
     * 适配BI5到6版本用户类型，将数据分析用户（5）和数据处理用户（6）转换为BI设计用户
     * @param platformTypeList
     */
    private static void adaptUserType(List<Integer> platformTypeList) {
        if (isBi6Version() && (platformTypeList.remove(new Integer(5)) | platformTypeList.remove(new Integer(6))) && !platformTypeList.contains(3)) {
            platformTypeList.add(3);
        }
    }

    private static boolean isBi6Version() {
        ProductVersion version = ProductVersionManager.getInstance().getVersionByModuleName("bi");
        return null != version && StringUtils.equals("6.0", version.getVersion());
    }

    /**
     * 随机生成强密码
     *
     * @param len
     * @return
     */
    public static String getRandomPassword(int len) {
        // 密码字典
        char[] str = {
                'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
                'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
                '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '~', '!', '@', '#', '$', '%', '^', '-', '+'
        };
        StringBuffer pwd = null;
        boolean flag = false;
        while (!flag) {
            int a = 0, b = 0, c = 0, d = 0;
            pwd = new StringBuffer();
            for (int i = 0; i < len; i++) {
                int rand = (int) (Math.random() * str.length);
                pwd.append(str[rand]);
                if (rand < 26) {
                    a++;
                } else if (rand < 52) {
                    b++;
                } else if (rand < 62) {
                    c++;
                } else {
                    d++;
                }
            }
            flag = (a * b * c * d != 0);
        }
        return pwd.toString();
    }
}
